#!/bin/bash
# -*- mode: shell-script; coding: utf-8 -*-
#
# Copyright 2002-2014 Cendio AB.
# For more information, see http://www.cendio.com

#
# This script is normally sourced, but can also be
# executed for testing.
#

if [ -z "${TLPREFIX}" ]; then
    echo "Cannot work without TLPREFIX. Exiting."
    exit 1
fi

source ${TLPREFIX}/libexec/functions

if [ -z "${TLSESSIONDATA}" ]; then
    echo "TLSESSIONDATA is not set. Are you in a proper ThinLinc session?"
    exit 1
fi

TLSTATE=startup

# Set a standard pointer instead of the default X
xsetroot -cursor_name left_ptr

# Set a background color and/or image
eval `"${TLPREFIX}/bin/tl-config" -e color=/sessionstart/background_color -e image=/sessionstart/background_image` 
if [ -n "${color}" -a -n "${image}" ]; then
    ${TLPREFIX}/libexec/tl-startup-bg --image="${image}" --color="${color}" &
elif [ -n "${image}" ]; then
    ${TLPREFIX}/libexec/tl-startup-bg --image="${image}" &
elif [ -n "${color}" ]; then
    ${TLPREFIX}/libexec/tl-startup-bg --color="${color}" &
else
    # If nothing is configured, show a white background
    ${TLPREFIX}/libexec/tl-startup-bg --color="#ffffff" &
fi
STARTUPBG_PID=$!

# Avoid empty sessions while Openbox is building font caches by
# building the font cache explicitly while also providing visual
# feedback if it takes time.
${TLPREFIX}/libexec/tl-fontcache

# Start a temporary Window Manager
${TLPREFIX}/libexec/openbox &

# Wait for WM to start
if ! wait_for_wm; then
    echo "$0: Timeout waiting for window manager to start"
fi

# Create a file where we dump the current status for the feedback
# application to read.
# (We cannot use a fifo because of potential blocking)
feedback_file="${TLSESSIONDATA}/tl-run-xstartup.$$"

# Start the feedback application
${TLPREFIX}/libexec/tl-run-xstartup-feedback ${feedback_file} &
FEEDBACK_PID=$!

# Set up a fd to the status tile
exec 23> ${feedback_file}

# Build a list of files to process
i=0
declare -a startup_files
shopt -s nullglob
for fname in "${TLPREFIX}/etc/xstartup.d"/* ; do
    # Ignore common backup file names
    case `basename "${fname}"` in
        *~|\#*|,*|_\$*|*$)
            continue
            ;;
    esac

    startup_files[$i]="${fname}"
    i=$[ $i + 1 ]
done

total=${#startup_files[@]}

i=0
for fname in "${startup_files[@]}" ; do
    desc=`"${TLPREFIX}/libexec/tl-xstartup-desc" "${fname}"`

    if [ -n "${desc}" ]; then
        echo "Running $fname ($desc)" >&2
    else
        echo "Running $fname" >&2
        desc="${fname}"
    fi

    # Update gauge
    echo $i $total "${desc}" 2>/dev/null >&23
    i=$[ $i + 1 ]

    case "${fname}" in
        *.sh) source "${fname}" ;;
        *)           "${fname}" ;;
    esac
done

# Save the environment
${TLPREFIX}/bin/tl-env -s

# Stop feedback application
echo "" 2>/dev/null >&23

# Close and remove the status file.
exec 23>&-
rm -f ${feedback_file}

# Terminate the feedback application
kill $FEEDBACK_PID 2>/dev/null
wait $FEEDBACK_PID >/dev/null 2>&1

# Stop the Window Manager
echo "Stopping initial window manager..."
${TLPREFIX}/libexec/openbox --exit
echo "Done."

# Terminate the startup background process
# FIXME: Note that the background might be briefly visible when
#        logging out. If the user has resized after this point,
#        the background seen during logout is the wrong size.
kill $STARTUPBG_PID 2>/dev/null
wait $STARTUPBG_PID >/dev/null 2>&1

TLSTATE=running
