/*
 * Copyright 2023 Cendio AB.
 * For more information, see http://www.cendio.com
 */

window.addEventListener("load", popupHandlers);

function popupHandlers() {

    let close_buttons = document.getElementsByClassName('popup_close');
    for (let i = 0; i < close_buttons.length; i++) {
        close_buttons[i].
            addEventListener("click", (event) => closePopup(event));
    }

    // Allow focus only in popup window
    try {
        const popup_window = document.getElementById('popup_window');
        const keyboardFocusable =
            'a[href], button, input, textarea, select, details, [tabindex]';
        document.querySelectorAll(keyboardFocusable).forEach((elem) => {
            if (popup_window.contains(elem)) return;
            elem.setAttribute("tabindex", "-1");
        });
    } catch (exc) {
        // Do nothing
    }

}

function closePopup(event) {
    let container = document.getElementById('popup_container');

    // Fade out and then close popup when animation is done
    container.addEventListener('animationend', navigateOutOfPopup,
                               { once: true });
    container.classList.add('closing');

    // Fallback to ensure popup closes even if animations are disabled
    setTimeout(navigateOutOfPopup, 500);
}

function navigateOutOfPopup() {
    window.location = window.location.pathname;
}

// This function is called directly from the HTML
function handlePopupFadeIn() {
    /* We only want to fade in the popup if opening the popup from a link.
       This is indicated by "details" being part of the search params in the
       URL. Otherwise, the popup should simply remain open and not fade in. */
    const details = new URLSearchParams(window.location.search).get('details');
    if (details) {
        let container = document.getElementById("popup_container");
        if (container) { container.classList.add("opening"); }
    }
}
