/*
 * Copyright 2023 Cendio AB.
 * For more information, see http://www.cendio.com
 */

window.addEventListener("load", textareaHandlers);

function textareaHandlers() {
    // Select all textareas with the "focused-rows" attribute.
    // These are text areas that changes size when focused.
    let textareas = document.querySelectorAll("textarea[focused-rows]");
    textareas.forEach((textarea) => setupFocusResize(textarea));
}

function setupFocusResize(textarea) {
    const initialRows = parseInt(textarea.getAttribute("rows"));
    const focusedRows = parseInt(textarea.getAttribute("focused-rows"));

    const textareaStyles = getComputedStyle(textarea);
    const initialHeight = parseFloat(textareaStyles.height);
    const lineHeight = parseFloat(textareaStyles.lineHeight);
    const boxSizing = textareaStyles.boxSizing;
    const originalTransitionDuration = textareaStyles.transitionDuration;

    // These are also needed if boxSizing is "border-box"
    const padding = parseFloat(textareaStyles.paddingTop); 
    const borderWidth = parseFloat(textareaStyles.borderWidth);
    const extraHeight = (padding + borderWidth) * 2;

    // CSS transitions for a given property requires the a change from a
    // defined value to another defined value. A height of "auto" doesn't
    // smoothly transition to a specific pixel-height, so let's initialize it.
    textarea.style.height = initialHeight + "px";

    // Expand textarea on focus
    textarea.addEventListener("focus", () => {
        // Use line-height and the number of rows to calculate the elem height.
        let focusedHeight = focusedRows * lineHeight;
        if (boxSizing == "border-box") focusedHeight += extraHeight;
        textarea.style.transitionDuration = "0.2s"
        textarea.style.height = focusedHeight + "px";
        textarea.addEventListener('transitionend', () => {
            textarea.style.transitionDuration = originalTransitionDuration;
        });
    });

    // Collapse on blur
    textarea.addEventListener("blur", () => {
        let blurredHeight = initialRows * lineHeight;
        if (boxSizing == "border-box") blurredHeight += extraHeight;
        textarea.style.transitionDuration = "0.2s"
        textarea.style.height = blurredHeight + "px";
        textarea.addEventListener('transitionend', () => {
            textarea.style.transitionDuration = originalTransitionDuration;
        });
    });
}

function toggle(id) {
    let el = document.getElementById(id);
    el.style.display = (el.style.display != 'none' ? 'none' : '' );

    // Toggling elements might reveal more textareas
    let textareas = el.querySelectorAll("textarea[focused-rows]");
    textareas.forEach((textarea) => setupFocusResize(textarea));
}
